<?php

namespace Drupal\jfu\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\json_field\Plugin\Field\FieldWidget\JSONTextareaWidget;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\Environment;

/**
 * Plugin implementation of the 'json_component_widget' widget.
 *
 * @FieldWidget(
 *   id = "json_component_widget",
 *   module = "jfu",
 *   label = @Translation("Component"),
 *   description = @Translation("Define all components."),
 *   field_types = {
 *     "json",
 *     "json_native",
 *     "json_native_binary"
 *   },
 * )
 */

class JSONComponentWidget extends JSONTextareaWidget {
  
  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $components = [
      'quote' => 'quote',
      'accordion' => 'accordion',
      'banner' => 'banner',
      'block' => 'block',
      'tabs' => 'tabs',
      'embed' => 'embed',
      'gallery' => 'gallery',
      'simple_card' => 'simple_card',
      'content_columns' => 'content_columns',
    ];

    $components_config = [
      'banner' => [
        'directory' => 'jfu-images',
        'max_size' => '',
        'max_dimensions' => [
          'width' => '',
          'height' => ''
        ],
      ],
      'tabs' => [
        'directory' => 'jfu-images',
        'max_size' => '',
        'max_dimensions' => [
          'width' => '',
          'height' => ''
        ],
      ],
      'simple_card' => [
        'directory' => 'jfu-images',
        'max_size' => '',
        'max_dimensions' => [
          'width' => '',
          'height' => ''
        ],
      ],
      'content_columns' => [
        'directory' => 'jfu-images',
        'max_size' => '',
        'max_dimensions' => [
          'width' => '',
          'height' => ''
        ],
      ],
    ];

    return [
      'components' => $components,
      'components_config' => $components_config,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    if (!isset($form['#is_jfu'])) {
      $form['#is_jfu'] = 'field--widget-json-component-widget';
    }

    $components = $this->getSetting('components');
    $cmd = DRUPAL_ROOT . '/' . drupal_get_path('module', 'jfu') . '/component_models/';
    $jfu_components = [];
    
    foreach ($components as $name) {
      if ($name !== 0) {
        $ufc = $cmd . $name . '/' . $name . '.json';
        $json_string = file_get_contents($ufc);
        $jfu_components[$name] = Json::decode($json_string);
      }
    }

    $field_storage_definition = $items->getFieldDefinition()->getFieldStorageDefinition();
    $field_name = $field_storage_definition->getName();
    $form['#attached']['drupalSettings']['jfu_components'][$field_name] = $jfu_components;
    
    $element['value'] = $element + [
      '#type' => 'textarea',
      '#default_value' => $items[$delta]->value,
      '#attributes' => [
        'target-weight' => $delta,
      ],
    ];

    if (!$items->isEmpty() && !is_null($items[$delta]->value)) {
      $form['#attached']['drupalSettings']['jfu'][$field_name][$delta] = '"' . $delta . '": ' . $items[$delta]->value;
    }
    
    return $element;
  }
  
  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $components = $this->getSetting('components');
    $components_config = $this->getSetting('components_config');
    $options = [];
    foreach ($components as $key => $name) {
      $options[$key] = ucfirst(str_replace('_', ' ', $key));
    }

    $element['components'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Components list'),
      '#options' => $options,
      '#default_value' => $components,
      '#required' => TRUE,
      '#multiple' => TRUE,
    ];

    $element['components_config'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Components configuration'),
    ];

    foreach ($components_config as $key_config => $config) {
      $element['components_config'][$key_config] = [
        '#type' => 'details',
        '#title' => $this->t('Config ' . $key_config),
        '#states' => [
          'visible' => [
            ':input[value="' . $key_config . '"]' => ['checked' => TRUE],
          ],
        ],
      ];

      $element['components_config'][$key_config]['directory'] = [
        '#type' => 'textfield',
        '#default_value' => $config['directory'],
        '#title' => t('Upload directory'),
        '#description' => t("A directory relative to Drupal's files directory where uploaded images will be stored."),
        '#states' => [
          'visible' => [
            ':input[value="' . $key_config . '"]' => ['checked' => TRUE],
          ],
        ],
      ];

      $default_max_size = format_size(Environment::getUploadMaxSize());
      $element['components_config'][$key_config]['max_size'] = [
        '#type' => 'textfield',
        '#default_value' => $config['max_size'],
        '#title' => t('Maximum file size'),
        '#description' => t('If this is left empty, then the file size will be limited by the PHP maximum upload size of @size.', ['@size' => $default_max_size]),
        '#maxlength' => 20,
        '#size' => 10,
        '#placeholder' => $default_max_size,
        '#states' => [
          'visible' => [
            ':input[value="' . $key_config . '"]' => ['checked' => TRUE],
          ],
        ],
      ];

      $element['components_config'][$key_config]['max_dimensions'] = [
        '#type' => 'item',
        '#title' => t('Maximum dimensions'),
        '#description' => t('Images larger than these dimensions will be scaled down.'),
        '#states' => [
          'visible' => [
            ':input[value="' . $key_config . '"]' => ['checked' => TRUE],
          ],
        ],
      ];

      $element['components_config'][$key_config]['max_dimensions']['width'] = [
        '#title' => t('Width'),
        '#title_display' => 'invisible',
        '#type' => 'number',
        '#default_value' => (empty($config['max_dimensions']['width'])) ? '' : $config['max_dimensions']['width'],
        '#size' => 8,
        '#maxlength' => 8,
        '#min' => 1,
        '#max' => 99999,
        '#placeholder' => t('width'),
        '#field_suffix' => ' x ',
        '#states' => [
          'visible' => [
            ':input[value="' . $key_config . '"]' => ['checked' => TRUE],
          ],
        ],
        '#prefix' => '<div class="form--inline clearfix">',
      ];

      $element['components_config'][$key_config]['max_dimensions']['height'] = [
        '#title' => t('Height'),
        '#title_display' => 'invisible',
        '#type' => 'number',
        '#default_value' => (empty($config['max_dimensions']['height'])) ? '' : $config['max_dimensions']['height'],
        '#size' => 8,
        '#maxlength' => 8,
        '#min' => 1,
        '#max' => 99999,
        '#placeholder' => t('height'),
        '#field_suffix' => t('pixels'),
        '#states' => [
          'visible' => [
            ':input[value="' . $key_config . '"]' => ['checked' => TRUE],
          ],
        ],
        '#suffix' => '</div>',
      ];
    }

    return $element;
  }
  
  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    
    $items = [];
    foreach ($this->getSetting('components') as $component) {
      if ($component !== 0) {
        $items[] = ucfirst(str_replace('_', ' ', $component));
      }
    }

    $list_components = [
      '#theme' => 'item_list',
      '#list_type' => 'ul',
      '#title' => $this->t('Allowed components'),
      '#items' => $items,
    ];
    
    $summary[] = $list_components;

    return $summary;
  }

}
